@@ -7,17 +7,18 @@ from collections import defaultdict
 from itertools import count
 
 import capstone
-import cffi
 import cle
 import networkx
 import pyvex
-from . import Analysis
 
+from . import Analysis
+from ..knowledge_plugins.cfg.memory_data import MemoryDataSort
 from ..knowledge_base import KnowledgeBase
 from ..sim_variable import SimMemoryVariable, SimTemporaryVariable
 
 l = logging.getLogger(name=__name__)
 
+
 #
 # Exceptions
 #
@@ -350,19 +351,19 @@ class SymbolManager(object):
                     symbol_name = symbol_name[ : symbol_name.index('@') ]
 
                 # check the type...
-                if symbol.type == cle.Symbol.TYPE_FUNCTION:
+                if symbol.type == cle.SymbolType.TYPE_FUNCTION:
                     # it's a function!
                     unique_symbol_name = self.get_unique_symbol_name(symbol_name)
                     label = FunctionLabel(self.binary, unique_symbol_name, addr)
-                elif symbol.type == cle.Symbol.TYPE_OBJECT:
+                elif symbol.type == cle.SymbolType.TYPE_OBJECT:
                     # it's an object
                     unique_symbol_name = self.get_unique_symbol_name(symbol_name)
                     label = ObjectLabel(self.binary, unique_symbol_name, addr)
-                elif symbol.type == cle.Symbol.TYPE_NONE:
+                elif symbol.type == cle.SymbolType.TYPE_NONE:
                     # notype
                     unique_symbol_name = self.get_unique_symbol_name(symbol_name)
                     label = NotypeLabel(self.binary, unique_symbol_name, addr)
-                elif symbol.type == cle.Symbol.TYPE_SECTION:
+                elif symbol.type == cle.SymbolType.TYPE_SECTION:
                     # section label
                     # use a normal label instead
                     if not name:
@@ -372,6 +373,12 @@ class SymbolManager(object):
                 else:
                     raise Exception('Unsupported symbol type %s. Bug Fish about it!' % symbol.type)
 
+            else:
+                raise Exception("the symbol %s is not owned by the main object. Try reload the project with"
+                                "\"auto_load_libs=False\". If that does not solve the issue, please report to GitHub."
+                                % symbol.name
+                                )
+
         elif (addr is not None and addr in self.cfg.functions) or is_function:
             # It's a function identified by angr's CFG recovery
 
@@ -410,7 +417,7 @@ class SymbolManager(object):
 
 
 class Operand(object):
-    def __init__(self, binary, insn_addr, insn_size, capstone_operand, operand_str, mnemonic, syntax=None):
+    def __init__(self, binary, insn_addr, insn_size, capstone_operand, operand_str, mnemonic, operand_offset, syntax=None):
         """
         Constructor.
 
@@ -419,6 +426,7 @@ class Operand(object):
         :param capstone_operand:
         :param str operand_str: the string representation of this operand
         :param str mnemonic: Mnemonic of the instruction that this operand belongs to.
+        :param int operand_offset: offset of the operand into the instruction.
         :param str syntax: Provide a way to override the default syntax coming from `binary`.
         :return: None
         """
@@ -429,8 +437,8 @@ class Operand(object):
         self.insn_size = insn_size
         self.operand_str = operand_str
         self.mnemonic = mnemonic
+        self.operand_offset = operand_offset
         self.syntax = self.binary.syntax if syntax is None else syntax
-
         self.type = None
         self.size = capstone_operand.size
 
@@ -509,7 +517,8 @@ class Operand(object):
                     s.append(base)
 
                 if self.index and self.scale:
-                    s.append('+')
+                    if s:
+                        s.append('+')
                     s.append("(%s * %d)" % (CAPSTONE_REG_MAP[self.project.arch.name][self.index], self.scale))
 
                 if disp:
@@ -524,11 +533,15 @@ class Operand(object):
                 asm = " ".join(s)
 
                 # we need to specify the size here
-                if 'dword' in self.operand_str.lower():
+                if self.size == 16:
+                    asm = 'xmmword ptr [%s]' % asm
+                elif self.size == 8:
+                    asm = 'qword ptr [%s]' % asm
+                elif self.size == 4:
                     asm = 'dword ptr [%s]' % asm
-                elif 'word' in self.operand_str.lower():
+                elif self.size == 2:
                     asm = 'word ptr [%s]' % asm
-                elif 'byte' in self.operand_str.lower():
+                elif self.size == 1:
                     asm = 'byte ptr [%s]' % asm
                 else:
                     raise BinaryError('Unsupported memory operand size for operand "%s"' % self.operand_str)
@@ -600,7 +613,7 @@ class Operand(object):
                     sort = 'call'
                 else:
                     sort = 'absolute'
-                self.binary.register_instruction_reference(self.insn_addr, imm, sort, self.insn_size)
+                self.binary.register_instruction_reference(self.insn_addr, imm, sort, self.operand_offset)
 
         elif self.type == OP_TYPE_MEM:
 
@@ -620,7 +633,7 @@ class Operand(object):
                 self.disp_label = self.binary.symbol_manager.new_label(addr=baseaddr)
                 self.disp_label_offset = self.disp - baseaddr
 
-                self.binary.register_instruction_reference(self.insn_addr, self.disp, 'absolute', self.insn_size)
+                self.binary.register_instruction_reference(self.insn_addr, self.disp, 'absolute', self.operand_offset)
 
     def _imm_to_ptr(self, imm, operand_type, mnemonic):  # pylint:disable=no-self-use,unused-argument
         """
@@ -696,8 +709,17 @@ class Instruction(object):
 
         self.labels = [ ]
 
+        operand_offsets = [ ]
+        for operand in capstone_instr.operands:
+            if operand.type == capstone.CS_OP_IMM:
+                operand_offsets.append(capstone_instr.imm_offset)
+            elif operand.type == capstone.CS_OP_MEM:
+                operand_offsets.append(capstone_instr.disp_offset)
+            else:
+                operand_offsets.append(None)
+
         if self.addr is not None:
-            self._initialize(capstone_instr.operands)
+            self._initialize(capstone_instr.operands, operand_offsets)
 
     #
     # Overridden predefined instructions
@@ -783,13 +805,10 @@ class Instruction(object):
             for i, op in enumerate(self.operands):
                 op_asm = op.assembly()
                 if op_asm is not None:
-                    if op.type == OP_TYPE_IMM:
+                    if op.type in (OP_TYPE_IMM, OP_TYPE_MEM):
                         all_operands[i] = op_asm
-                    elif op.type == OP_TYPE_MEM:
-                        if self.binary.syntax == 'intel':
-                            all_operands[i] = all_operands[i][ : all_operands[i].index('ptr') + 3] + " [" + op_asm + "]"
-                        elif self.binary.syntax == 'at&t':
-                            all_operands[i] = op_asm
+                    else:
+                        raise BinaryError("Unsupported operand type %d." % op.type)
 
                     if self.capstone_operand_types[i] == capstone.CS_OP_IMM:
                         if mnemonic.startswith('j') or mnemonic.startswith('call') or mnemonic.startswith('loop'):
@@ -817,7 +836,7 @@ class Instruction(object):
     # Private methods
     #
 
-    def _initialize(self, capstone_operands):
+    def _initialize(self, capstone_operands, operand_offsets):
         """
         Initialize this object
 
@@ -827,9 +846,9 @@ class Instruction(object):
         if self.addr is None:
             raise InstructionError('self.addr must be specified')
 
-        self._initialize_operands(capstone_operands)
+        self._initialize_operands(capstone_operands, operand_offsets)
 
-    def _initialize_operands(self, capstone_operands):
+    def _initialize_operands(self, capstone_operands, operand_offsets):
         """
 
         :return:
@@ -837,9 +856,10 @@ class Instruction(object):
 
         all_operands = split_operands(self.op_str)
         capstone_operands = capstone_operands[ - len(all_operands) : ] # sometimes there are more operands than expected...
+        operand_offsets = operand_offsets[ - len(all_operands) : ]
 
-        for operand, operand_str in zip(capstone_operands, all_operands):
-            self.operands.append(Operand(self.binary, self.addr, self.size, operand, operand_str, self.mnemonic))
+        for operand, operand_str, offset in zip(capstone_operands, all_operands, operand_offsets):
+            self.operands.append(Operand(self.binary, self.addr, self.size, operand, operand_str, self.mnemonic, offset))
 
 class BasicBlock(object):
     """
@@ -1202,11 +1222,11 @@ class Data(object):
         """
         self.size = new_size
 
-        if self.sort == 'string':
+        if self.sort == MemoryDataSort.String:
             self.null_terminated = False # string without the null byte terminator
             self._content[0] = self._content[0][ : self.size]
 
-        elif self.sort == 'pointer-array':
+        elif self.sort == MemoryDataSort.PointerArray:
             pointer_size = self.binary.project.arch.bytes
 
             if self.size % pointer_size != 0:
@@ -1231,7 +1251,7 @@ class Data(object):
         :return: None
         """
 
-        self.sort = 'unknown'
+        self.sort = MemoryDataSort.Unknown
         content = self.binary.fast_memory_load(self.addr, self.size, bytes)
         self.content = [ content ]
 
@@ -1250,7 +1270,7 @@ class Data(object):
                 labels = self.binary.symbol_manager.addr_to_label[addr]
 
                 for label in labels:
-                    if self.sort == 'pointer-array' and addr % (self.project.arch.bytes) != 0:
+                    if self.sort == MemoryDataSort.PointerArray and addr % (self.project.arch.bytes) != 0:
                         # we need to modify the base address of the label
                         base_addr = addr - (addr % (self.project.arch.bytes))
                         label.base_addr = base_addr
@@ -1274,7 +1294,7 @@ class Data(object):
         if self.skip:
             return s
 
-        if self.sort == 'string':
+        if self.sort == MemoryDataSort.String:
 
             if symbolized:
                 ss = [ ]
@@ -1320,7 +1340,7 @@ class Data(object):
                 s += "\t.{directive} \"{str}\"".format(directive=directive, str=string_escape(self.content[0]))
             s += '\n'
 
-        elif self.sort == 'pointer-array':
+        elif self.sort == MemoryDataSort.PointerArray:
 
             if self.binary.project.arch.bits == 32:
                 directive = '.long'
@@ -1359,13 +1379,13 @@ class Data(object):
                 for label in self.content:
                     s += "\t%s %s\n" % (directive, label.operand_str)
 
-        elif self.sort == 'segment-boundary':
+        elif self.sort == MemoryDataSort.SegmentBoundary:
 
             if symbolized:
                 for _, label in self.labels:
                     s += "\t%s\n" % str(label)
 
-        elif self.sort == 'integer':
+        elif self.sort == MemoryDataSort.Integer:
             # display it as bytes only when there are references pointing to the middle
             content = [ ]
 
@@ -1436,7 +1456,7 @@ class Data(object):
             s += "\n".join(content)
             s += "\n"
 
-        elif self.sort == 'fp':
+        elif self.sort == MemoryDataSort.FloatingPoint:
             # we have to display it as bytes...
             # TODO: switch to "ten byes" whenever time permits
             content = []
@@ -1507,8 +1527,7 @@ class Data(object):
             if self.size is None or self._initial_content is None and self.sort is None:
                 raise BinaryError('You must at least specify size, initial_content, and sort.')
 
-
-            if self.sort == 'pointer-array':
+            if self.sort == MemoryDataSort.PointerArray:
 
                 lbl = DataLabel(self.binary, -1, name=self.name)
                 self.labels.append((0, lbl))
@@ -1547,14 +1566,14 @@ class Data(object):
                         label = addr
                     self._content.append(label)
 
-            elif self.sort in ('string', 'unknown', 'integer'):
+            elif self.sort in {MemoryDataSort.String, MemoryDataSort.Unknown, MemoryDataSort.Integer}:
 
                 lbl = DataLabel(self.binary, -1, name=self.name)
                 self.labels.append((0, lbl))
 
                 self._content = [ self._initial_content ]
 
-            elif self.sort == 'segment-boundary':
+            elif self.sort == MemoryDataSort.SegmentBoundary:
                 label = self.binary.symbol_manager.new_label(self.addr)
                 self.labels.append((self.addr, label))
                 self._content = []
@@ -1568,7 +1587,7 @@ class Data(object):
             self.sort = self.memory_data.sort
 
             # Symbolize the content
-            if self.sort == 'pointer-array':
+            if self.sort == MemoryDataSort.PointerArray:
                 # read out the address
                 pointer_size = self.project.arch.bytes
                 pointers = self.size // pointer_size
@@ -1597,7 +1616,7 @@ class Data(object):
                         # it's a static pointer. we should use the original pointer value.
                         self._content.append(addr)
 
-            elif self.sort == 'string':
+            elif self.sort == MemoryDataSort.String:
                 data = self.binary.fast_memory_load(self.addr, self.size, bytes)
                 if data[-1] == 0:
                     self.null_terminated = True
@@ -1607,17 +1626,17 @@ class Data(object):
 
                 self._content = [data]
 
-            elif self.sort == 'integer':
+            elif self.sort == MemoryDataSort.Integer:
                 data = self.binary.fast_memory_load(self.addr, self.size, bytes)
                 self._content = [ data ]
 
-            elif self.sort == 'segment-boundary':
+            elif self.sort == MemoryDataSort.SegmentBoundary:
                 label = self.binary.symbol_manager.new_label(self.addr)
                 self.labels.append((self.addr, label))
 
                 self._content = [ ]
 
-            elif self.sort == 'fp':
+            elif self.sort == MemoryDataSort.FloatingPoint:
                 # floating-point integers
                 # Python has some trouble in dealing with floating point numbers
                 # just store them as bytes
@@ -1633,7 +1652,7 @@ class Data(object):
                     self._content = []
 
 
-class Relocation(object):
+class Relocation:
     def __init__(self, addr, ref_addr, sort):
         self.addr = addr
         self.ref_addr = ref_addr
@@ -1886,54 +1905,12 @@ class Reassembler(Analysis):
             return closest_region
         return False, None
 
-    def register_instruction_reference(self, insn_addr, ref_addr, sort, insn_size):
+    def register_instruction_reference(self, insn_addr, ref_addr, sort, operand_offset):
 
         if not self.log_relocations:
             return
 
-        addr = insn_addr
-        if sort == 'jump':
-            addr += 1
-        elif sort == 'call':
-            addr += 1
-        elif sort == 'absolute':
-            # detect it...
-            ptr_size = self.project.arch.bytes
-            for i in range(0, insn_size):
-                # an absolute address is used
-                if insn_size - i >= ptr_size:
-                    ptr = self.fast_memory_load(insn_addr + i, ptr_size, int, endness='Iend_LE')
-                    if ptr == ref_addr:
-                        addr += i
-                        break
-
-                # an absolute address of 4 bytes is used
-                # e.g. AMD64:
-                #      mov r8, offset 0x400070
-                #      49 c7 c0 xx xx xx xx
-                if ptr_size == 8 and insn_size - i >= 4:
-                    ptr = self.fast_memory_load(insn_addr + i, 4, int, endness='Iend_LE')
-                    if ptr == ref_addr:
-                        addr += i
-                        break
-
-                # an relative offset is used, and size of the offset is 4
-                # e.g. AMD64:
-                #      mov rax, 0x600100
-                #      48 8b 05 xx xx xx
-                if insn_size - i >= 4:
-                    ptr = self.fast_memory_load(insn_addr + i, 4, int, endness='Iend_LE')
-                    if (ptr + insn_addr + insn_size) == ref_addr:
-                        addr += i
-                        break
-            else:
-                l.warning('Cannot find the absolute address inside instruction at %#x. Use the default address.',
-                          insn_addr
-                          )
-
-        else:
-            raise BinaryError('Unsupported sort "%s" in register_instruction_reference().' % sort)
-
+        addr = insn_addr + operand_offset
         r = Relocation(addr, ref_addr, sort)
 
         self._relocations.append(r)
@@ -2161,22 +2138,24 @@ class Reassembler(Analysis):
         # there is a single function referencing them
         cgcpl_memory_data = self.cfg.memory_data.get(cgc_package_list.addr, None)
         cgcea_memory_data = self.cfg.memory_data.get(cgc_extended_application.addr, None)
+        refs = self.cfg.kb.xrefs
 
         if cgcpl_memory_data is None or cgcea_memory_data is None:
             return False
 
-        if len(cgcpl_memory_data.refs) != 1:
+        if len(refs.get_xrefs_by_dst(cgcpl_memory_data.addr)) != 1:
             return False
-        if len(cgcea_memory_data.refs) != 1:
+        if len(refs.get_xrefs_by_dst(cgcea_memory_data.addr)) != 1:
             return False
 
         # check if the irsb addresses are the same
-        if next(iter(cgcpl_memory_data.refs))[0] != next(iter(cgcea_memory_data.refs))[0]:
+        if next(iter(refs.get_xrefs_by_dst(cgcpl_memory_data.addr))).block_addr != \
+                next(iter(refs.get_xrefs_by_dst(cgcea_memory_data.addr))).block_addr:
             return False
 
-        insn_addr = next(iter(cgcpl_memory_data.refs))[2]
+        insn_addr = next(iter(refs.get_xrefs_by_dst(cgcpl_memory_data.addr))).ins_addr
         # get the basic block
-        cfg_node = self.cfg.get_any_node(insn_addr, anyaddr=True)
+        cfg_node = self.cfg.model.get_any_node(insn_addr, anyaddr=True)
         if not cfg_node:
             return False
 
@@ -2249,6 +2228,16 @@ class Reassembler(Analysis):
         :return: None
         """
 
+        # determine if the binary is compiled against glibc
+        is_glibc = False
+        for dep in self.project.loader.main_object.deps:
+            if dep.lower() in {'libc.so.6', 'libc.so'}:
+                is_glibc = True
+                break
+        if is_glibc:
+            self.remove_unnecessary_stuff_glibc()
+
+    def remove_unnecessary_stuff_glibc(self):
         glibc_functions_blacklist = {
             '_start',
             '_init',
@@ -2343,7 +2332,7 @@ class Reassembler(Analysis):
             self._section_alignments[section.name] = alignment
 
         l.debug('Generating CFG...')
-        cfg = self.project.analyses.CFG(normalize=True, resolve_indirect_jumps=True, collect_data_references=True,
+        cfg = self.project.analyses.CFG(normalize=True, resolve_indirect_jumps=True, data_references=True,
                                         extra_memory_regions=[(0x4347c000, 0x4347c000 + 0x1000)],
                                         data_type_guessing_handlers=[
                                             self._sequence_handler,
@@ -2753,11 +2742,11 @@ class Reassembler(Analysis):
 
             # execute the single basic block and see how the value is used
             base_graph = networkx.DiGraph()
-            candidate_node = self.cfg.get_any_node(candidate.irsb_addr)  # type: angr.analyses.cfg_node.CFGNode
+            candidate_node = self.cfg.model.get_any_node(candidate.irsb_addr)  # type: angr.analyses.cfg_node.CFGNode
             if candidate_node is None:
                 continue
             base_graph.add_node(candidate_node)
-            tmp_kb = KnowledgeBase(self.project, self.project.loader.main_object)
+            tmp_kb = KnowledgeBase(self.project)
             cfg = self.project.analyses.CFGEmulated(kb=tmp_kb,
                                                     starts=(candidate.irsb_addr,),
                                                     keep_state=True,
