@@ -62,7 +62,8 @@ class VariableRecoveryFastState(VariableRecoveryStateBase):
 
         return state
 
-    def merge(self, others: Tuple['VariableRecoveryFastState'], successor=None) -> Tuple['VariableRecoveryFastState',bool]:
+    def merge(self, others: Tuple['VariableRecoveryFastState'],
+              successor=None) -> Tuple['VariableRecoveryFastState',bool]:
         """
         Merge two abstract states.
 
@@ -91,7 +92,7 @@ class VariableRecoveryFastState(VariableRecoveryStateBase):
 
         merged_typevars = self.typevars
         merged_typeconstraints = self.type_constraints.copy()
-        delayed_typeconstraints = self.delayed_type_constraints.copy()
+        delayed_typeconstraints = self.delayed_type_constraints.copy().clean()
         for other in others:
             merged_typevars = merged_typevars.merge(other.typevars)
             merged_typeconstraints |= other.type_constraints
@@ -100,7 +101,7 @@ class VariableRecoveryFastState(VariableRecoveryStateBase):
 
         merge_occurred |= self.typevars != merged_typevars
         merge_occurred |= self.type_constraints != merged_typeconstraints
-        merge_occurred |= self.delayed_type_constraints != self.delayed_type_constraints
+        merge_occurred |= self.delayed_type_constraints != delayed_typeconstraints
 
         # add subtype constraints for all replacements
         for v0, v1 in self.phi_variables.items():
@@ -169,7 +170,7 @@ class VariableRecoveryFast(ForwardAnalysis, VariableRecoveryBase):  #pylint:disa
     """
 
     def __init__(self, func, func_graph=None, max_iterations=2, low_priority=False, track_sp=True,
-                 func_args: Optional[List[SimVariable]]=None):
+                 func_args: Optional[List[SimVariable]]=None, store_live_variables=False):
         """
 
         :param knowledge.Function func:  The function to analyze.
@@ -183,7 +184,7 @@ class VariableRecoveryFast(ForwardAnalysis, VariableRecoveryBase):  #pylint:disa
         if not func.block_addrs_set or func.startpoint is None:
             raise AngrVariableRecoveryError("Function %s is empty." % repr(func))
 
-        VariableRecoveryBase.__init__(self, func, max_iterations)
+        VariableRecoveryBase.__init__(self, func, max_iterations, store_live_variables)
         ForwardAnalysis.__init__(self, order_jobs=True, allow_merging=True, allow_widening=False,
                                  graph_visitor=function_graph_visitor)
 
@@ -204,6 +205,7 @@ class VariableRecoveryFast(ForwardAnalysis, VariableRecoveryBase):  #pylint:disa
         self._analyze()
 
         # cleanup (for cpython pickle)
+        self.downsize()
         self._ail_engine = None
         self._vex_engine = None
 
@@ -292,8 +294,6 @@ class VariableRecoveryFast(ForwardAnalysis, VariableRecoveryBase):  #pylint:disa
         :return:
         """
 
-        input_state = state  # make it more meaningful
-
         if type(node) is ailment.Block:
             # AIL mode
             block = node
@@ -310,9 +310,9 @@ class VariableRecoveryFast(ForwardAnalysis, VariableRecoveryBase):  #pylint:disa
         #         l.debug('Merging input state of node %#x with the previous state.', node.addr)
         #         input_state, _ = prev_state.merge((input_state,), successor=node.addr)
 
-        state = input_state.copy()
+        state = state.copy()
         state.block_addr = node.addr
-        self._instates[node.addr] = input_state
+        # self._instates[node.addr] = state
 
         if self._node_iterations[node.addr] >= self._max_iterations:
             l.debug('Skip node %#x as we have iterated %d times on it.', node.addr, self._node_iterations[node.addr])
@@ -320,12 +320,13 @@ class VariableRecoveryFast(ForwardAnalysis, VariableRecoveryBase):  #pylint:disa
 
         self._process_block(state, block)
 
-        self._outstates[node.addr] = state
-
         self._node_iterations[node.addr] += 1
         self.type_constraints |= state.type_constraints
         self.var_to_typevar.update(state.typevars._typevars)
 
+        state.downsize()
+        self._outstates[node.addr] = state
+
         return True, state
 
     def _intra_analysis(self):
@@ -335,11 +336,13 @@ class VariableRecoveryFast(ForwardAnalysis, VariableRecoveryBase):  #pylint:disa
         self.variable_manager['global'].assign_variable_names(labels=self.kb.labels)
         self.variable_manager[self.function.addr].assign_variable_names()
 
-        for addr, state in self._outstates.items():
-            self.variable_manager[self.function.addr].set_live_variables(addr,
-                                                                         state.register_region,
-                                                                         state.stack_region
-                                                                         )
+        if self._store_live_variables:
+            for addr, state in self._outstates.items():
+                self.variable_manager[self.function.addr].set_live_variables(
+                    addr,
+                    state.downsize_region(state.register_region),
+                    state.downsize_region(state.stack_region),
+                )
 
     #
     # Private methods
