@@ -242,7 +242,9 @@ class SimState(PluginHub):
             p.init_state()
 
     def __getstate__(self):
-        s = { k:v for k,v in self.__dict__.items() if k not in ('inspect', 'regs', 'mem')}
+        # Don't pickle attributes for plugins. These will be pickled
+        # through self._active_plugins.
+        s = { k:v for k,v in self.__dict__.items() if k not in self._active_plugins.keys() }
         s['_active_plugins'] = { k:v for k,v in s['_active_plugins'].items() if k not in ('inspect', 'regs', 'mem') }
         return s
 
@@ -382,7 +384,7 @@ class SimState(PluginHub):
     T = TypeVar("T")
     def _inspect_getattr(self, attr: str, default_value: T):
         if self.supports_inspect:
-            if hasattr(self.inspect, attr):
+            if self.inspect.action_attrs_set and hasattr(self.inspect, attr):
                 return getattr(self.inspect, attr)
 
         return default_value
